/*
 * @copyright        © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module            ep.fn.holdOnTop
 */

define("ep/fn/holdontop", [
    "jquery",
    "ep"
], function($, ep) {

    var win = $(window),
        elementStack = [],
        eventTimeout = [],

        holdOnTopUnity = function(i) {
            if (eventTimeout[i]) {
                return;
            }

            eventTimeout[i] = true;

            var self = this,
                o = self.options,
                elem = self.element,
                body = $('body'),
                bodyPos = {
                    top: window.pageYOffset,
                    left: window.pageXOffset
                },
                setCSS = {
                    position: 'fixed'
                },
                elemPos,
                bodyHeight,
                bodyWidth;

            if (!self.placeholder) {
                self.placeholder = $('<div>').insertBefore(elem);
            }

            self.placeholder.hide();

            elem.attr('style', self.restore)
                .removeClass('holdOnTop');

            elemPos = elem.offset();

            if (o == 'bottom') {
                bodyHeight = body.outerHeight(true);

                elemPos.bottom = bodyHeight - elem.outerHeight(true) - elemPos.top + parseFloat(elem.css("padding-bottom")) + parseFloat(elem.css("margin-bottom"));
                bodyPos.bottom = bodyHeight - win.height() - bodyPos.top;
                setCSS.left = '0';

            } else if (o == 'right') {
                bodyWidth = body.outerWidth(true);

                elemPos.right = bodyWidth - elem.outerWidth(true) - elemPos.left;
                bodyPos.right = bodyWidth - win.width() - bodyPos.left;
            }

            if (bodyPos[o] > elemPos[o]) {
                setCSS[o] = 0 + 'px';
                self.placeholder
                    .width(elem.outerWidth(true))
                    .height(elem.outerHeight(true))
                    .show();

                elem.css(setCSS)
                    .addClass('holdOnTop');
            }

            setTimeout(function() {
                eventTimeout[i] = false;
            }, 10);
        },

        holdOnTop = function(i) {
            var self = this,
                o = self.options,
                elem = self.element,
                body = $('body'),
                placeholder = (self.placeholder || (self.placeholder = $('<div>').insertBefore(elem)))
                    .hide(),

                elemPos = elem
                    .attr('style', self.restore)
                    .offset(),

                bodyPos = {
                    top: window.pageYOffset,
                    left: window.pageXOffset
                },

                setCSS = {
                    position: 'fixed'
                };

            if (o == 'bottom') {
                var bodyHeight = body.outerHeight(true);

                elemPos.bottom = bodyHeight - elem.outerHeight(true) - elemPos.top;
                bodyPos.bottom = bodyHeight - win.height() - bodyPos.top;
                setCSS.left = '0';

            } else if (o == 'right') {
                var bodyWidth = body.outerWidth(true);

                elemPos.right = bodyWidth - elem.outerWidth(true) - elemPos.left;
                bodyPos.right = bodyWidth - win.width() - bodyPos.left;
            }

            if (bodyPos[o] > elemPos[o]) {
                setCSS[o] = 0 + 'px';
                self.placeholder
                    .width(elem.outerWidth(true))
                    .height(elem.outerHeight(true))
                    .show();

                elem.css(setCSS)
                    .addClass('holdOnTop');

            } else if (bodyPos[o] < elemPos[o]) {
                elem.removeClass('holdOnTop');
            }
        },

        onEvent = function() {
            var fn = ep.config.unity ? holdOnTopUnity : holdOnTop;

            $.each(elementStack, fn);
        };

    ep.fn.holdOnTop = function(options) {
        this.each(function() {
            var elem = $(this);

            elementStack.push({
                element: $(this),
                options: options || 'top',
                restore: elem.attr('style') || ""
            });
        });

        onEvent();

        return this;
    };

    win.on('scroll resize accordionactivate accordionchange', onEvent);

    return ep;
});