/*global define*/
/*jslint vars:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/mediagallery/ui/filemanagerdialog-test.html',
    'jquery',
    'de_epages/mediagallery/ui/filemanagerdialog',
    '$ready!'
], function (QUnit, fixture, $, de_epages) {
    'use strict';

    QUnit.module("de_epages/mediagallery/ui/filemanagerdialog", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("de_epages/mediagallery/ui/filemanagerdialog", function () {

        // fake data for ajax request to initialise the file manager
        var initArray = [{
            "currentGallerySize": "1344",
            "children": [{
                "files": [{
                    "width": null,
                    "extension": ".swf",
                    "name": "promo.swf",
                    "path": "/promo.swf",
                    "height": null,
                    "directory": "/MediaGallery",
                    "modified": 1200994146,
                    "size": 67089,
                    "fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/promo.swf",
                    "selectable": 1,
                    "mimetype": "application/x-shockwave-flash"
                }],
                "children": [{
                    "files": [],
                    "children": [],
                    "metadata": {
                        "displayname": "neu",
                        "type": "MediaGallery",
                        "path": "/neu"
                    },
                    "data": "neu",
                    "state": null
                }],
                "metadata": {
                    "displayname": "MediaGallery",
                    "type": "MediaGallery",
                    "path": ""
                },
                "data": "MediaGallery",
                "state": "closed"
            }, {
                "files": [],
                "children": [{
                    "files": [],
                    "children": [],
                    "metadata": {
                        "displayname": "Kunst, Kultur und Bildung",
                        "type": "Header",
                        "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                    },
                    "data": "ArtAndEducation",
                    "state": null
                }],
                "metadata": {
                    "displayname": "Kopfzeilenbilder",
                    "type": "Header",
                    "path": "/StyleGroups/Branches/StyleGroups"
                },
                "data": "HeaderImages",
                "state": "closed"
            }, {
                "files": [],
                "children": [{
                    "files": [],
                    "children": [],
                    "metadata": {
                        "displayname": "Kunst, Kultur und Bildung",
                        "type": "Background",
                        "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                    },
                    "data": "ArtAndEducation",
                    "state": null
                }],
                "metadata": {
                    "displayname": "Hintergrundbilder",
                    "type": "Background",
                    "path": "/StyleGroups/Branches/StyleGroups"
                },
                "data": "BackgroundImages",
                "state": "closed"
            }, {
                "files": [],
                "children": [{
                    "files": [],
                    "children": [],
                    "metadata": {
                        "displayname": "Gebäude",
                        "type": "ImageDB",
                        "path": "/ImageDBGroups/Buildings"
                    },
                    "data": "Buildings",
                    "state": null
                }],
                "metadata": {
                    "displayname": "{ImageDB}",
                    "type": "ImageDB",
                    "path": "ImageDBGroups"
                },
                "data": "ImageDB",
                "state": "closed"
            }],
            "maxGallerySize": "102400"
        }];

        $.ajax = function (o) {
            var result = $.Deferred();

            if (o.data.ViewAction === "JSONInitMediaGallery") {
                result.resolve(initArray);
            }

            return result;
        };

        // create widget
        var filemanager = $("#filemanager").mediagalleryUiFilemanagerdialog();

        QUnit.equal(
            filemanager.mediagalleryUiFilemanagerdialog("Instance") instanceof $.Widget,
            true,
            "Filemanager dialog widget is created and instance of jQuery.Widget"
        );

        $.each(['_create', 'open', 'close', 'destroy', '_createFilemanager', 'resetFilemanager', 'getSelectedElements'], function (i, fn) {

            QUnit.equal(
                $.isFunction($.ui.mediagalleryUiFilemanagerdialog.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );

        });

        // open filemanager
        filemanager.trigger("click");

        QUnit.equal(
            $('.ui-dialog:visible').length,
            1,
            "File manager was opened by clicking the button"
        );

        QUnit.equal(
            filemanager.mediagalleryUiFilemanagerdialog("getCurrentUsage"),
            "1344",
            "Get the current gallery size"
        );

        // close dialog
        filemanager.mediagalleryUiFilemanagerdialog("resetFilemanager");
        filemanager.mediagalleryUiFilemanagerdialog("close");

        QUnit.equal(
            $('.ui-dialog:visible').length,
            0,
            "File manager was closed"
        );

        // open dialog
        filemanager.mediagalleryUiFilemanagerdialog("open");

        QUnit.equal(
            $('.ui-dialog:visible').length,
            1,
            "File manager was opened by open method"
        );

        // destroy widget and trigger click to check whether the dialog was destroyed
        filemanager.mediagalleryUiFilemanagerdialog("destroy");
        filemanager.trigger("click");

        QUnit.equal(
            filemanager.data("mediagalleryUiFilemanagerdialog"),
            undefined,
            "Widget is successfully destroyed"
        );


        QUnit.start();

    });
});
