/*globals define*/

define([
    'testsuite/qunit',
    'jquery',
    'de_epages/presentation/ui/uploader',
    'testsuite/qunit-helper',

    '$ready!'
], function (QUnit, $, de_epages, helper) {

    QUnit.module("de_epages/presentation/ui/uploader", {
        setup: function () {
            $('<input id="uploader" type="file" />').appendTo('#qunit-fixture');
        }
    });

    QUnit.asyncTest("prototype methods defined", function () {

        QUnit.expect(19);

        $.each(['_create', '_setOption', '_upload', '_uploadReplace', '_uploadCancel', '_uploadProgress', '_uploadSuccess', '_uploadError', '_uploadComplete', '_progressOpen', '_progressClose', '_progressCreate', '_errorOpen', '_errorClose', '_errorCreate', '_replaceOpen', '_replaceClose', '_replaceCreate', 'destroy'], function (i, fn) {

            QUnit.equal(
                $.isFunction($.ui.presentationUiUploader.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );

        });

        QUnit.start();
    });

    QUnit.asyncTest("de_epages/presentation/ui/uploader", function () {
        var fileForUpload = 'fileForUpload.jpg',
            arrResult = [{Strings: [fileForUpload]}, {}],
            objResult = arrResult[0],
            uploadList = [],
            uploader,

            /* START - fake file input element in upload method */
            _upload = $.ui.presentationUiUploader.prototype._upload,
            _errorClose = $.ui.presentationUiUploader.prototype._errorClose,
            fakeSwitchNode,

            fakeSwitch = function (self, fake) {
                if (!fakeSwitchNode) {
                    fakeSwitchNode = self.elem[0];
                }

                if (fake) {
                    self.elem = $();
                    self.elem[0] = helper.fake('HTMLInputElement', {
                        type: 'file',
                        files: [
                            helper.fake('File', fileForUpload, 'image/jpeg', 2048)
                        ]
                    });
                    self.elem.length = 1;
                } else {
                    self.elem = $(fakeSwitchNode);
                }
            };

        $.ui.presentationUiUploader.prototype._upload = function () {
            var node = this.elem[0],
                result;

            fakeSwitch(this, true);
            result = _upload.apply(this, arguments);
            fakeSwitch(this, false);

            return result;
        };

        $.ui.presentationUiUploader.prototype._errorClose = function () {
            var result;

            fakeSwitch(this, false);
            result = _errorClose.apply(this, arguments);
            fakeSwitch(this, true);

            return result;
        };
        /* END - fake file input element in upload method */


        /* START - fake ajax request */
        $.ajax = function (o) {
            var jqXHR = $.Deferred();

            if (o.data && o.data.ViewAction === 'JSONUnicodeConverter') {
                jqXHR.resolve(objResult);
            } else if (o.data && o.data.ChangeAction === 'SaveImage') {
                jqXHR.notify($.extend($.Event("progress"), {
                    "lengthComputable": true,
                    "position": 2048,
                    "total": 2048,
                    "loaded": 2048,
                    "item": 1,
                    "itemTotal": 1,
                    "itemLoaded": 0,
                    "data": {
                        "success": {"upload": "Ok"},
                        "item": {"name": "NewFile", "fileName": fileForUpload, "fileSize": 2048, "fileType": "image/jpeg"}
                    }
                }), "success-item");

                setTimeout(function () {
                    jqXHR.resolve({});
                }, 250);
            } else {
                jqXHR.reject();
            }

            return jqXHR.promise();
        }
        /* END - fake ajax request */

        uploader = $('#uploader')
                .presentationUiUploader({
                    url:        '../ajaxtransport-test.json',
                    data:       {
                        ChangeAction:       'SaveImage',
                        ObjectID:           0,
                        SCALE_WIDTH_m:      200,
                        SCALE_WIDTH_s:      100,
                        SCALE_HEIGHT_s:     100,
                        SCALE_WIDTH_h:      150,
                        SCALE_HEIGHT_h:     150,
                        SCALE_WIDTH_xs:     50,
                        SCALE_HEIGHT_xs:    33,
                        SCALE_NAME_m:       'ImageMedium',
                        SCALE_NAME_s:       'ImageSmall',
                        SCALE_NAME_h:       'ImageHotDeal'
                    },
                    multiple:   true,
                    accept:     'image/*',
                    callback:   function( data, statusText ){
                        if (data[0]!== undefined && data[0].success) {
                            // push successful uploads in upload list (needed for upload tests)
                            uploadList.push(data[0]);
                        }
                    },
                    exists: [
                        'existingFile.jpg'
                    ]
                });


        QUnit.equal(
            uploader.data('uiPresentationUiUploader') instanceof $.Widget,
            true,
            "Uploader widget is created and instance of jQuery.Widget"
        );

        // test some options
        uploader.presentationUiUploader('option', 'style', 'icon');

        QUnit.ok(uploader.presentationUiUploader('option', 'style') === 'icon', 'option \'style\' set to \'icon\'');

        uploader.presentationUiUploader('option', 'style', 'big-icon');

        QUnit.ok(uploader.presentationUiUploader('option', 'style') === 'big-icon', 'option \'style\' set to \'big-icon\'');

        uploader.presentationUiUploader('option', 'big', false);

        QUnit.ok(uploader.presentationUiUploader('option', 'big') === false, 'option \'big\' set to false');

        uploader.presentationUiUploader('option', 'big', true);

        QUnit.ok(uploader.presentationUiUploader('option', 'big') === true, 'option \'big\' set to true');

        // add and remove class
        uploader.presentationUiUploader('addClass', 'myFancyClass');

        QUnit.equal(
            uploader.hasClass('myFancyClass'),
            true,
            '"myFancyClass" was added'
        );

        uploader.presentationUiUploader('removeClass', 'myFancyClass');

        QUnit.equal(
            uploader.hasClass('myFancyClass'),
            false,
            '"myFancyClass" was removed'
        );

        // standard upload
        uploader.trigger('change');

        QUnit.ok($('.de_epages-presentationUiUploader-progressList').is(':hidden') === false, 'progress dialog visible (upload in progress)');

        // wait for first request
        setTimeout(function(){

            QUnit.ok($('.de_epages-presentationUiUploader-progressList').is(':hidden') === true, 'progress dialog invisible (upload done)');

            // replace file
            uploader.data('uiPresentationUiUploader').options.exists = [fileForUpload,'existingFile.jpg','existingFile2.jpg','existingFile3.jpg'];
            uploader.trigger('change');

            // close dialog
            $('.epDialog:eq(1) button:first').trigger('click');

            // wait for second request
            setTimeout(function(){

                // start and cancel upload
                uploader.trigger('change');

                // close dialog
                $('.epDialog:eq(1) button:last').trigger('click');

                // next lines would be needed when testing the error case - but this cannot be tested, because it cannot be faked
                //objResult = arrResult[1];
                //$('#uploader').trigger('change');

                // -- Test: error dialog
                uploader.data('uiPresentationUiUploader')._errorOpen();

                QUnit.ok($('.de_epages-presentationUiUploader-errorList:last').is(':hidden') === false, 'error dialog is open');

                $('#uploader').data('uiPresentationUiUploader')._errorClose();

                QUnit.ok($('.de_epages-presentationUiUploader-errorList:last').is(':hidden') === true, 'error dialog is closed');

                // -- Test: error dialog

                QUnit.ok(uploadList.length === 1, uploadList.length + ' successful uploads');

                // destroy widget
                uploader.presentationUiUploader('destroy');

                QUnit.ok(uploader.data('uiPresentationUiUploader') === undefined, 'Uploader widget was successfully destroyed');

                QUnit.start();

            }, 300);
        }, 300);
    });
});